#include "matrix.h"
#include "mex.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "bvBasicFunctions.h"

/*************************
 * bvToolbox  Copyright (C) 2013  PIERRE Fabien (read license.txt)
 * Auteur 	: Fabien PIERRE.
 * Version	:	1.1.
 * Date	    : 30 mai 2013.
*/

/*I3=bvProximalOperator(Im_bruitee, 25, sigma);*/
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* Declarations.*/
    double *result, *gradient_x, *gradient_y,*Im1, *Im2,*Im, *dv0, *G, *G2, *G1, *g;
    int height, width, ndim, N, longueur, itt, i;
    int *dims;
    double lambda;
    
    /* check for proper number of arguments */
    if(nrhs!=3) {
        mexErrMsgIdAndTxt("bvToolbox:bvProximalOperator:nrhs","3 inputs required.");
    }
    if(nlhs!=1) {
        mexErrMsgIdAndTxt("bvToolbox:bvProximalOperator:nlhs","One output required.");
    }
    
    /*gestion entrees-sorties.*/
    ndim= mxGetNumberOfDimensions(prhs[0]);
    dims= mxGetDimensions(prhs[0]);
    height = dims[0];
    width = dims[1];
    
    N = (int)(*(mxGetPr(prhs[1])));
    lambda = (double)(*(mxGetPr(prhs[2])));
    g = (double*)mxGetPr(prhs[0]);
    
    plhs[0]= mxCreateNumericArray(ndim, dims, mxDOUBLE_CLASS, mxREAL);
    result= (double*)mxGetPr(plhs[0]);
    
    /*Calcul de la divergence.*/
    
    longueur=height*width;
    Im=(double*)mxCalloc(longueur, sizeof(double));
    Im1=(double*)mxCalloc(longueur, sizeof(double));
    Im2=(double*)mxCalloc(longueur, sizeof(double));
    G=(double*)mxCalloc(longueur, sizeof(double));
    G1=(double*)mxCalloc(longueur, sizeof(double));
    G2=(double*)mxCalloc(longueur, sizeof(double));
    dv0=(double*)mxCalloc(longueur, sizeof(double));
    
    /* Iterations de l'agorithme de Chambolle. */
    for (itt = 1.0; itt <= N; itt++) {
        divergence(Im1, Im2, dv0, height, width);
        for (i = 0; i < longueur; i++) {
            G[i] = dv0[i] - g[i] / lambda;
        }
        
        gradient(G, G1, G2, height, width);
        
        
        
        for (i = 0; i < longueur; i++) {
            G[i]=sqrt(G1[i] * G1[i] + G2[i] * G2[i]);
        }
        
        for (i = 0; i < longueur; i++) {
            Im2[i] = (Im2[i] + 0.2 * G2[i]) / (1.0 + 0.2 * G[i]);
            Im1[i] = (Im1[i] + 0.2 * G1[i]) / (1.0 + 0.2 * G[i]);
        }
    }
    
    divergence(Im1, Im2, Im, height, width);
    for (i = 0; i <longueur; i++) {
        result[i] = g[i] - lambda * Im[i];
    }
    
    /*Liberation de la memoire.*/
    mxFree(Im);
    mxFree(Im1);
    mxFree(Im2);
    mxFree(G);
    mxFree(G1);
    mxFree(G2);
    mxFree(dv0);
    
    return;
}
