#include "matrix.h"
#include "mex.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "bvBasicFunctions.h"

/*************************
 * bvToolbox  Copyright (C) 2013  PIERRE Fabien (read license.txt)
 * Auteur 	: Fabien PIERRE.
 * Version	:	1.1.
 * Date	    : 30 mai 2013.
*/

//result= inpaintingPrimalDual(Data, tau, theta, sigma, lambda, itterations, mask)
//I3=inpaintingPrimalDual(Data, 5, 1, 0.02, 1000,1000, Mask);
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* Declarations.*/
    double *result, *x, *x_bar, *y1, *y2, *t, *g_x, *g_y, *x_1, *Im, *g, *mask;
    int height, width, ndim, longueur, pix, itterations,i;
    int *dims;
    double lambda, normi, sigma, tau, theta;
    
    /* Vrification du nombre d'arguments.*/
    if(nrhs!=7) {
        mexErrMsgIdAndTxt("MyToolbox:inpaintingPrimalDual:nrhs","7 inputs required.");
    }
    if(nlhs!=1) {
        mexErrMsgIdAndTxt("MyToolbox:inpaintingPrimalDual:nlhs","One output required.");
    }
    
    /*gestion entrees-sorties.*/
    ndim= mxGetNumberOfDimensions(prhs[0]);
    dims= mxGetDimensions(prhs[0]);
    height = dims[0];
    width = dims[1];
    
    tau = (double)(*(mxGetPr(prhs[1])));
    theta = (double)(*(mxGetPr(prhs[2])));
    lambda = (double)(*(mxGetPr(prhs[4])));
    sigma = (double)(*(mxGetPr(prhs[3])));
    Im = (double*)mxGetPr(prhs[0]);
    itterations = (int)(*(mxGetPr(prhs[5])));
    mask = (double*)mxGetPr(prhs[6]);
    
    plhs[0]= mxCreateNumericArray(ndim, dims, mxDOUBLE_CLASS, mxREAL);
    g_x= (double*)mxGetPr(plhs[0]);
    
    /*Allocation mmoire*/
    longueur=height*width;
    x= (double*)mxCalloc(longueur,sizeof(double));
    x_bar = (double*)mxCalloc(longueur,sizeof(double));
    y1= (double*)mxCalloc(longueur,sizeof(double));
    y2= (double*)mxCalloc(longueur,sizeof(double));
    t = (double*)mxCalloc(longueur,sizeof(double));
    g_y = (double*)mxCalloc(longueur,sizeof(double));
    
    /* Iterations de l'agorithme primal-dual. */
    for(pix=0; pix<longueur; pix++)
    {
        x[pix]=Im[pix];
        x_bar[pix]=Im[pix];
    }
    
    for(i=0; i<itterations; i++)
    {
        gradient(x_bar,g_x, g_y, height, width);
        
        for(pix=0; pix<longueur; pix++)
        {
            g_x[pix]= y1[pix] - sigma * g_x[pix];
            g_y[pix]= y2[pix] - sigma * g_y[pix];
        }
        
        for(pix=0; pix<longueur; pix++)
        {
			normi= sqrt(g_x[pix] * g_x[pix] + g_y[pix] * g_y[pix]);
			if(1 > normi)
			{
				normi= 1;
			}
            y1[pix]=g_x[pix] / normi;
            y2[pix]=g_y[pix] / normi;
        }
        
        divergence( y1, y2 ,g_x, height, width);
        
        for( pix=0; pix<longueur; pix++)
        {
            g_x[pix]=x[pix]-tau*g_x[pix];
        }
        
        for(pix=0; pix<longueur; pix++)
        {
            if(mask[pix]!=0)
            {
                normi= tau*lambda;
                g_x[pix]=(g_x[pix]+ normi*Im[pix]) /(1+normi);
            }
        }
        
        for(pix=0; pix<longueur; pix++)
        {
            normi= g_x[pix];
            x_bar[pix]= normi + theta * (normi - x[pix]);
            x[pix]=normi;
        }
    }
    
    /*Liberation de la memoire.*/
    mxFree(x);
    mxFree(x_bar );
    mxFree(y1 );
    mxFree(y2 );
    mxFree(t );
    mxFree(g_y );
    
    return;
}
