#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "mex.h"
#include "matrix.h"
#include <math.h>
#include <cuda_runtime.h>

#define THREADS_PER_BLOCK	512
#define VERBOSE 1

/*************************
 * bvToolbox  Copyright (C) 2013  PIERRE Fabien (read license.txt)
 * Auteur 	: Fabien PIERRE.
 * Version	:	1.1.
 * Date	    : 30 mai 2013.
*/

/*Kernel gradient*/
__global__ void
        gradient( const float *image_initiale, float *gradient_x, float *gradient_y,int height, int width)
{
    int i = blockDim.x * blockIdx.x + threadIdx.x;
    
    int lin= i % height;
    int col= (i-lin)/height;
    
    if(lin==(height-1))
    {
        gradient_x[lin+col*height]=0;
    }
    else
    {
        gradient_x[lin+col*height]=
                image_initiale[lin+(col)*height+1]
                -image_initiale[lin+col*height];
    }
    if((col==(width-1)))
    {
        gradient_y[lin+col*height]= 0;
    }
    else
    {
        gradient_y[lin+col*height]=
                image_initiale[lin+(col+1)*height]
                -image_initiale[lin+col*height];
    }
}

/*Kernel divergence*/
__global__ void
        divergence( const float *gradient_x, const float *gradient_y, float *result, int height, int width)
{
    int i = blockDim.x * blockIdx.x + threadIdx.x;
    
    int lin= i % height;
    int col= (i-lin)/height;
    
    float x_1;
    float x_2;
    if(lin==0)
    {
        x_1=gradient_x[lin+height*col];
    }
    else if (lin==(height-1))
    {
        x_1=-gradient_x[lin+height*col-1];
    }
    else
    {
        x_1=gradient_x[lin+height*col]
                -gradient_x[lin+height*col-1];
    }
    
    if (col==0)
    {
        x_2=gradient_y[lin+height*col];
    }
    else if (col==(width-1))
    {
        x_2=-gradient_y[lin+height*(col-1)];
    }
    else
    {
        x_2=gradient_y[lin+height*col]
                -gradient_y[lin+height*(col-1)];
    }
    result[lin+height*col]= x_1+x_2;
}


/* FONCTION_1*/
__global__ void
        fonction_1(const float *Im, float *x, float *x_bar, float *g_x, float *g_y, float *y1, float *y2 )
{
    int pix = blockDim.x * blockIdx.x + threadIdx.x;
    x[pix]=Im[pix];
    x_bar[pix]=Im[pix];
    g_x[pix]=0;
    g_y[pix]=0;
    y1[pix]=0;
    y2[pix]=0;
}


__global__ void
        fonction_2(float *racine_d, float *y1, float *y2, float *g_x, float *g_y, float *vari_d)
{
    int pix = blockDim.x * blockIdx.x + threadIdx.x;
    g_x[pix]= y1[pix] - vari_d[4] * g_x[pix];
    g_y[pix]= y2[pix] - vari_d[4] * g_y[pix];
    racine_d[pix]= (float)(sqrt( g_x[pix] * g_x[pix]+ g_y[pix]*g_y[pix]));
}


__global__ void
        max(   float *g , float *m_max, int longueur)
{

    __shared__ float max[THREADS_PER_BLOCK];
    
    int arrayIndex = THREADS_PER_BLOCK*blockIdx.x + threadIdx.x;
    if(arrayIndex >= longueur){
        return ;
    }
    
    max[threadIdx.x] = g[arrayIndex];
    __syncthreads();
    int nTotalThreads = blockDim.x;
    int milieu = 0;
    float temp= 0.0f;
    
    while(nTotalThreads > 1)
    {
        milieu = (nTotalThreads >> 1);
        
        if (threadIdx.x < milieu)
        {
            temp = max[threadIdx.x + milieu];
            if (temp > max[threadIdx.x]){
                max[threadIdx.x] = temp;
            }
        }
        __syncthreads();
        
        nTotalThreads = (nTotalThreads >> 1);
    }
    
    if (threadIdx.x == 0)
    {
        g[blockDim.x*blockIdx.x]= max[0];
        m_max[0] = max[0];
    }
    
}

__global__ void
        proj( float *m_min)
{
    if(m_min[0] < 1.0f)
    {
        m_min[0]=1.0f;
    }
    else{
        m_min[0]=m_min[0];
    }
}

__global__ void
        group(   float *g )
{
    int i = blockDim.x * blockIdx.x + threadIdx.x;
    g[i] = g[blockDim.x*i];
}

__global__ void
        fonction_4(float *y1, float *y2, const float *g_x, const float *g_y, float *max_d)
{
    int pix = blockDim.x * blockIdx.x + threadIdx.x;
    float norm= (float)(sqrt( y1[pix] * y1[pix] + y2[pix] * y2[pix] ));
	if(norm > 1){
     y1[pix]= g_x[pix] / norm;
     y2[pix]= g_y[pix] / norm;
	}
	else{
     y1[pix]= g_x[pix];
     y2[pix]= g_y[pix];
	}
}

__global__ void
        fonction_5(float *x, float *vari, float *g_x, float *Im, float *mask, float *x_bar)
{
    int pix = blockDim.x * blockIdx.x + threadIdx.x;
    
    g_x[pix]=x[pix]-vari[2]*g_x[pix];
    
    float normi;
    
    if(mask[pix] > 0.5 )
    {
        normi= vari[2]*vari[3];
        g_x[pix]=(g_x[pix]+ normi*Im[pix]) /(1+normi);
    }
    
    normi= g_x[pix];
    x_bar[pix]= normi + vari[1] * (normi - x[pix]);
    x[pix]=normi;
    
}

//result= inpaintingPrimalDual(Data, tau, theta, sigma, lambda, itterations, mask)
//I3=inpaintingPrimalDual(Data, 5, 1, 0.02, 10000,300, Mask);
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    /* Declarations.*/
    double   *g_x, *Im,  *mask;
    int height, width, ndim, longueur, itterations;
    int *dims;
    double lambda, sigma, tau, theta;
    cudaError_t err ;
    
    /* Vérification du nombre d'arguments.*/
    if(nrhs!=7) {
        mexErrMsgIdAndTxt("MyToolbox:inpaintingPrimalDualGPU:nrhs","7 inputs required.");
    }
    if(nlhs!=1) {
        mexErrMsgIdAndTxt("MyToolbox:inpaintingPrimalDualGPU:nlhs","One output required.");
    }
    
    /*gestion entrees-sorties.*/
    ndim= mxGetNumberOfDimensions(prhs[0]);
    dims= (int*)mxGetDimensions(prhs[0]);
    height = dims[0];
    width = dims[1];
    
    tau = (double)(*(mxGetPr(prhs[1])));
    theta = (double)(*(mxGetPr(prhs[2])));
    lambda = (double)(*(mxGetPr(prhs[4])));
    sigma = (double)(*(mxGetPr(prhs[3])));
    Im = (double*)mxGetPr(prhs[0]);
    itterations = (int)(*(mxGetPr(prhs[5])));
    mask = (double*)mxGetPr(prhs[6]);
    
    plhs[0]= mxCreateNumericArray(ndim, (const mwSize *)dims, mxDOUBLE_CLASS, mxREAL);
    g_x= (double*)mxGetPr(plhs[0]);
    
    /*Allocation mémoire*/
    longueur=height*width;
    
    float *x_d = NULL;
    float *x_bar_d = NULL;
    float *y1_d = NULL;
    float *y2_d = NULL;
    float *t_d = NULL;
    float *g_x_d = NULL;
    float *g_y_d = NULL;
    float *Im_d = NULL;
    float *mask_d = NULL;
    float *vari_d = NULL;
    float *racine_d = NULL;
    
    size_t size= height*width*sizeof(float);
    
    err = cudaMalloc((void **)&x_d, size);
    if (err != cudaSuccess){
        mexPrintf( "probleme allocation memoire GPU (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaMalloc((void **)&x_bar_d, size);
    if (err != cudaSuccess){
        mexPrintf( "probleme allocation memoire GPU (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaMalloc((void **)&y1_d, size);
    if (err != cudaSuccess){
        mexPrintf( "probleme allocation memoire GPU (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaMalloc((void **)&y2_d, size);
    if (err != cudaSuccess){
        mexPrintf( "probleme allocation memoire GPU (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaMalloc((void **)&t_d, size);
    if (err != cudaSuccess){
        mexPrintf( "probleme allocation memoire GPU (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaMalloc((void **)&g_x_d, size);
    if (err != cudaSuccess){
        mexPrintf( "probleme allocation memoire GPU (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaMalloc((void **)&g_y_d, size);
    if (err != cudaSuccess){
        mexPrintf( "probleme allocation memoire GPU (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaMalloc((void **)&Im_d, size);
    if (err != cudaSuccess){
        mexPrintf( "probleme allocation memoire GPU (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaMalloc((void **)&mask_d, size);
    if (err != cudaSuccess){
        mexPrintf( "probleme allocation memoire GPU (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaMalloc((void **)&racine_d, size);
    if (err != cudaSuccess){
        mexPrintf( "probleme allocation memoire GPU (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaMalloc((void **)&vari_d, 5*sizeof(float));
    if (err != cudaSuccess){
        mexPrintf( "probleme allocation memoire GPU (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    float *max_h = (float*)malloc(sizeof(float));
    float *max_d;
    err = cudaMalloc((void **)&max_d, sizeof(float));
    if (err != cudaSuccess)
    {
        mexPrintf( "probleme alloc max_d (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    float *intermediaire= (float*)malloc(size);
    float *vari_h= (float*)malloc(5*sizeof(float));
    
    vari_h[0]= 0;
    vari_h[1]= (float)theta;
    vari_h[2]= (float)tau;
    vari_h[3]= (float)lambda;
    vari_h[4]= (float)sigma;
    
    for(int i = 0; i<longueur; i++){
        intermediaire[i]=(float)Im[i];
    }
    
	clock_t instant1= clock();
	
    err = cudaMemcpy(Im_d, intermediaire, size, cudaMemcpyHostToDevice);/*check*/
    if (err != cudaSuccess){
        mexPrintf( "probleme copie (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    for(int i = 0; i<longueur; i++){
        intermediaire[i]=(float)mask[i];
    }
    
    err = cudaMemcpy(mask_d, intermediaire, size, cudaMemcpyHostToDevice);/*check*/
    if (err != cudaSuccess){
        mexPrintf( "probleme copie (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaMemcpy(vari_d, vari_h, 5*sizeof(float) , cudaMemcpyHostToDevice);/*check*/
    if (err != cudaSuccess){
        mexPrintf( "probleme copie (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    int numElements= height*width;
    int blocksPerGrid = numElements / THREADS_PER_BLOCK  ;
			
    /* Iterations de l'agorithme primal-dual. */
    
    fonction_1<<<blocksPerGrid , THREADS_PER_BLOCK>>>(Im_d, x_d, x_bar_d, g_x_d, g_y_d, y1_d, y2_d );
    err = cudaGetLastError();
    if (err != cudaSuccess){
        mexPrintf( "probleme fonction 1 (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    for(int i=0; i<itterations; i++)
    {
        
        gradient<<<blocksPerGrid, THREADS_PER_BLOCK>>>(x_bar_d, g_x_d, g_y_d, height, width);
        err = cudaGetLastError();
        if (err != cudaSuccess){
            mexPrintf( "probleme gradient (error code %s)!\n", cudaGetErrorString(err));
            return ;
        }
        
        fonction_2<<<blocksPerGrid, THREADS_PER_BLOCK>>>(racine_d, y1_d, y2_d, g_x_d, g_y_d, vari_d);
        err = cudaGetLastError();
        if (err != cudaSuccess){
            mexPrintf( "probleme fonction 2 (error code %s)!\n", cudaGetErrorString(err));
            return ;
        }
        
        fonction_4<<<blocksPerGrid , THREADS_PER_BLOCK>>>(y1_d, y2_d, g_x_d, g_y_d, max_d);
        err = cudaGetLastError();
        
        if (err != cudaSuccess)
        {
            mexPrintf( "probleme fonction 4 (error code %s)!\n", cudaGetErrorString(err));
            return ;
        }
        
        /*CALCUL_DIVERGENCE
         * divergence( y1, y2 ,g_x, height, width);
         */
        divergence<<<blocksPerGrid, THREADS_PER_BLOCK>>>(y1_d, y2_d, g_x_d, height, width);
        err = cudaGetLastError();
        if (err != cudaSuccess){
            mexPrintf( "probleme divergence (error code %s)!\n", cudaGetErrorString(err));
            return ;
        }
        
        fonction_5<<<blocksPerGrid, THREADS_PER_BLOCK>>>(x_d, vari_d, g_x_d, Im_d, mask_d, x_bar_d);
        err = cudaGetLastError();
        if (err != cudaSuccess){
            mexPrintf( "probleme fonction 5 (error code %s)!\n", cudaGetErrorString(err));
            return ;
        }
        
    }
		
    err = cudaMemcpy(intermediaire, g_x_d, size, cudaMemcpyDeviceToHost);/*check*/{
        
        if (err != cudaSuccess){
            mexPrintf( "Failed to copy from device to host (error code %s)!\n", cudaGetErrorString(err));
            return ;
        }
    }
	
	if(VERBOSE){
    clock_t instant2= clock();
	mexPrintf("temps GPU = %f ms.\n", difftime(instant2, instant1));
	}
	
    /*Re converion en double.*/
    for(int i=0;  i<height*width; i++){
        g_x[i]= (double)intermediaire[i];
    }
    
    err = cudaFree(x_d);
    if (err != cudaSuccess){
        mexPrintf( "probleme liberation memoire (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaFree(x_bar_d);
    if (err != cudaSuccess){
        mexPrintf( "probleme liberation memoire (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaFree(y1_d);
    if (err != cudaSuccess){
        mexPrintf( "probleme liberation memoire (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaFree(y2_d);
    if (err != cudaSuccess){
        mexPrintf( "probleme liberation memoire (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaFree(t_d);
    if (err != cudaSuccess){
        mexPrintf( "probleme liberation memoire (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaFree(g_x_d);
    if (err != cudaSuccess){
        mexPrintf( "probleme liberation memoire (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaFree(g_y_d);
    if (err != cudaSuccess){
        mexPrintf( "probleme liberation memoire (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaFree(Im_d);
    if (err != cudaSuccess){
        mexPrintf( "probleme liberation memoire (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaFree(mask_d);
    if (err != cudaSuccess){
        mexPrintf( "probleme liberation memoire (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaFree(racine_d);
    if (err != cudaSuccess){
        mexPrintf( "probleme liberation memoire (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaFree(vari_d);
    if (err != cudaSuccess){
        mexPrintf( "probleme liberation memoire (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaFree(max_d);
    if (err != cudaSuccess){
        mexPrintf( "probleme liberation memoire (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    err = cudaDeviceReset();
    if (err != cudaSuccess){
        mexPrintf( "probleme Reset GPU (error code %s)!\n", cudaGetErrorString(err));
        return ;
    }
    
    
    /*Liberation de la memoire.*/
    
    free(vari_h);
    free(max_h);
    free(intermediaire);
    
    return;
}
