
% bvToolbox  Copyright (C) 2013  PIERRE Fabien (read license.txt)
% Auteur 	: Fabien PIERRE.
% Version	: 1.1
% Date      : 30 mai 2013.

%% Compilation des programmes.
setup
% setup clear
% setup gpu

%% Test des fonctions div et grad.
clc, clear all, close all

load lena

tic
[Gx,Gy]=gradient(Im);
D=divergence(Gx,Gy);
toc

figure
imagesc(Im)
colormap gray
axis square
axis off
title('Image originale.')

figure
imagesc(Gx)
colormap gray
axis square
axis off
title('Gradient.')

figure
imagesc(Gy)
colormap gray
axis square
axis off
title('Gradient.')

figure
imagesc(D)
colormap gray
axis square
axis off
title('Divergence.')

%% Creation de l'image masquee et du masque.
clc, clear all, close all

load lena

% pourcentage de pixels perdus.
d=75;

[u,v]=size(Im);
NM=u*v;
P=randperm(NM);
Mask=ones(size(Im));
Mask(P(1:floor(d*NM/100)))=0;
Data = Mask.* Im;

imagesc(Data)
colormap gray
axis square
axis off
title('Image masquee.')

figure
imagesc(Im)
colormap gray
axis square
axis off
title('Image originale.')

%% Test de impainting_FB.
%*** attention : lenteur**** Elapsed time is 251.947115 seconds.
tic
I3=inpainting_FB(Data,100,30,30,Mask);
toc

figure
imagesc(Data)
title('Image masquee.')
colormap gray
axis square
axis off

figure
imagesc(I3)
colormap gray
title('Image inpaintee.')
axis square
axis off


%% Test de inpaintingPrimalDual. (env. 30 sec.)
close all
clc

tic
I3=inpaintingPrimalDual_mex(Data, 10 , 1.4 , 0.01, 0.2, 60, Mask);
toc

figure
imagesc(Data)
title('Image masquee.')
colormap gray
axis square
axis off

figure
imagesc(I3)
colormap gray
title('Image inpaintee.')
axis square
axis off

%% Debruitage par operateur proximal (minimisation de la variation totale.)
close all
clc
load lena

sigma= 30;
Im_bruitee= Im+sigma*randn(size(Im));

tic
I3=bvProximalOperator(Im_bruitee, 30, 50);
toc

figure
imagesc(Im_bruitee)
colormap gray
axis square
title('Image bruitee.')
axis off
imwrite(uint8(adjust(Im_bruitee)), 'lena_bruitGaussienAdditif.png', 'PNG');

figure
imagesc(I3)
colormap gray
title('Image debruitee.')
axis square
axis off
imwrite(uint8(adjust(I3)), 'lena_DEbruitGaussienAdditif.png', 'PNG');

%% Forward-Backward et Beck Teboulle applique a l'inpainting.
clc, clear all, close all

load lena

h=length(Im(:));
Mask=zeros(size(Im));
R=randperm(h);
Mask(R(1:floor(h*0.25)))=1;

Data=Im.*Mask;

figure
imagesc(Data)
axis square
colormap gray
axis off

tic
I2=inpainting_FB_BT(Data,70,5,7,Mask);
toc

figure
imagesc(Data)
title('Image masquee.')
colormap gray
axis square
axis off

figure
imagesc(I2)
colormap gray
title('Image inpaintee.')
axis square
axis off


%% TVL1 bruit poivre et sel (env. 30 sec.)
close all
clc

load lena

sigma=100;
percent=25;
p=percent/100;
Data2=Im;
h=length(Im(:));
R=randperm(h);
Data2(R(1:floor(h*p)))=Data2(R(1:floor(h*p)))...
    +sigma*randn(1,floor(h*p));
IM= Data2>255;
Data2(IM)=255;
IM= Data2<0;
Data2(IM)=0;

tic
I2=TVL1(Data2,10 , 1.4 , 0.01, 1, 10);
toc

figure
imagesc(Data2)
colormap gray
axis square
title('Image bruitee.')
axis off
%imwrite(uint8(adjust(Data2)), 'lena_pfeffersaltz.png', 'PNG');

figure
imagesc(I2)
colormap gray
title('Image debruitee.')
axis square
axis off
%imwrite(uint8(adjust(I2)), 'lena_DEpfeffersaltz.png', 'PNG');

%% Deconvolution.
close all
clc

sigma=6;
[X,Y]= meshgrid(-255: 256,-255: 256);
gauss= exp(-(X.^2+Y.^2)/(sigma^2));
gauss= gauss/(sum(gauss(:)));

F_gauss=fft2(fftshift(gauss));

load lena

Data=real(ifft2(fft2(Im).*F_gauss)) + 1 *randn(size(Im));

tic
I3=deconvolutionPrimalDual(Data, 0.0021, 0.5, 90,  17, F_gauss);
toc

%I3= convolutionProximalOperator( F_gauss, Data, Data, 1000 );

figure
imagesc(Data)
axis square
colormap gray
title('Image convoluee bruitee.')
axis off
%imwrite(uint8(adjust(Data)), 'lena_conv.png', 'PNG');

figure
imagesc(I3)
axis square
colormap gray
title('Image restauree.')
axis off
%imwrite(uint8(adjust(I3)), 'lena_deconv.png', 'PNG');

%% test maximaux

clc, clear all, close all

load lena

% pourcentage de pixels perdus.
d=93;

[u,v]=size(Im);
NM=u*v;
P=randperm(NM);
Mask=ones(size(Im));
Mask(P(1:floor(d*NM/100)))=0;
Data = Mask.* Im;

imagesc(Data)
colormap gray
axis square
axis off
title('Image masquee.')

figure
imagesc(Im)
colormap gray
axis square
axis off
title('Image originale.')

tic
I3=inpaintingPrimalDual_mex(Data, 90 , 0.42 , 0.003, 10, 150, Mask);
toc

figure
imagesc(I3)
colormap gray
title('Image inpaintee.')
axis square
axis off

%% test2
clc, clear all, close all

load lena

[u,v]=gradient(Im);

g=sqrt(u.^2+v.^2);
a=sort(g(:));

[x,y]=meshgrid(-6:6,-6:6);
F=exp(-1/16*(x.^2+y.^2));
F=F/sum(F(:));

g=filter2(F,g);

M=g./max(a);

R=rand(size(M));
Mask=double(R<(M*1.5));

display(['Pixels perdus : ' , num2str(100*(1-sum(sum(Mask))/length(a))) , ' %']);

Data= Mask.*Im;

imagesc(Data)
colormap gray
axis square
axis off
title('Image masquee.')

figure
imagesc(Im)
colormap gray
axis square
axis off
title('Image originale.')

tic
I3=inpaintingPrimalDual_mex(Data, 90 , 0.42 , 0.003, 10, 300, Mask);
toc

figure
imagesc(I3)
colormap gray
title('Image inpaintee.')
axis square
axis off

