function setup(arg)
% setup Installe les fonctions de la toolbox.
%	'setup' pour installer la toolbox en version CPU.
%	'setup clear' pour desintaller.
%
%   'setup gpu' pour installer la toolbox en version parallelisee (GPU).
%   Pour les utilisateurs de Windows, si une erreur survient lors de 
%   l'intallation GPU, verifiez les points suivants :
%   - Vous disposez d'une carte graphique processeur NVidia, compatible
%       avec CUDA.
%   - Vous avez installe CUDA et nvcc apparait dans $PATH
%   - Vous avez installe une version recente de MS VisualC++. 
%     (tapez 'mex -setup' dans le prompt pour vous assurez qu'il apparait 
%     bien.)
%   - Vous disposez de Microsoft SDK.
%

% bvToolbox  Copyright (C) 2013  PIERRE Fabien (read license.txt)
% Auteur 	: Fabien PIERRE.
% Version	: 1.1
% Date      : 30 mai 2013.
%

cd ./librairie
if isunix
    if nargin==0
        clc
        
        addpath(pwd);
        cd ./bvM_Files
        addpath(pwd);
        cd ..
        cd ./bvData
        addpath(pwd);
        cd ..
        savepath;
        
        disp('Installation start ...')
        disp(' ')
        
        % compilation du fichier des fonctions de base.
        
        mex -c CFLAGS="\$CFLAGS -std=c99" -I'./bvBasicLib' bvBasicLib/bvBasicFunctions.c
        
        % compilation des fonctions.
        mex CFLAGS="\$CFLAGS -std=c99" bvBasicFunctions.o -I'./bvBasicLib' bvSrc_C/bvGradient.c -output bvGradient_mex
        mex CFLAGS="\$CFLAGS -std=c99" bvBasicFunctions.o -I'./bvBasicLib' bvSrc_C/bvDivergence.c -output bvDivergence_mex
        mex CFLAGS="\$CFLAGS -std=c99" bvBasicFunctions.o -I'./bvBasicLib' bvSrc_C/bvProximalOperator.c -output bvProximalOperator_mex
        mex CFLAGS="\$CFLAGS -std=c99" bvBasicFunctions.o -I'./bvBasicLib' bvSrc_C/inpaintingPrimalDual.c -output inpaintingPrimalDual_mex
        mex CFLAGS="\$CFLAGS -std=c99" bvBasicFunctions.o -I'./bvBasicLib' bvSrc_C/TVL1_mex.c -output TVL1_mex
        mex CFLAGS="\$CFLAGS -std=c99" bvBasicFunctions.o -I'./bvBasicLib' bvSrc_C/deconvolutionPrimalDual.c -output deconvolutionPrimalDual_mex
        
        % Si tout va bien :
        disp(' ')
        disp('Installation complete successfull !')
        disp(' ')
        disp('Vous disposez desormais des fonctions suivantes :')
        disp('bvGradient')
        disp('bvDivergence')
        disp('bvProximalOperator')
        disp('inpaintingPrimalDual')
        disp('deconvolutionPrimalDual')
        disp('TVL1_mex')
        disp(' ')
        disp('Amusez-vous bien !')
    elseif  strcmp(arg, 'clear')
        % nettoyage complet
        clc, clear all
        
        list=dir;
        for i=3:length(list)
            fname= list(i).name;
            if (strcmp(fname(end-1:end),'.c') || ...
                    strcmp(fname(end-1:end),'.h')|| ...
                    strcmp(fname(end-1:end),'.m')|| ...
                    strcmp(fname(end-2:end),'.cu')|| ...
                    strcmp(fname(end-3:end),'.mat')|| ...
                    strcmp(fname(end-3:end),'.bat')|| ...
                    isdir(fname))
                
            else
                delete(list(i).name);
            end
        end
        rmpath(pwd);
        cd ./bvM_Files
        rmpath(pwd);
        cd ..
        cd ./bvData
        rmpath(pwd);
        cd ..
        
    elseif  strcmp(arg, 'gpu')
        clc
        
        addpath(pwd);
        cd ./bvM_Files
        addpath(pwd);
        cd ..
        cd ./bvData
        addpath(pwd);
        cd ..
        savepath;
        
        disp('Installation start ...')
        disp(' ')
        
        % compilation du fichier des fonctions de base.
        mex -c CFLAGS="\$CFLAGS -std=c99" -I'./bvBasicLib' bvBasicLib/bvBasicFunctions.c
        
        % compilation des fonctions.
        mex CFLAGS="\$CFLAGS -std=c99" bvBasicFunctions.o -I'./bvBasicLib' bvSrc_C/bvGradient.c -output bvGradient_mex
        mex CFLAGS="\$CFLAGS -std=c99" bvBasicFunctions.o -I'./bvBasicLib' bvSrc_C/bvDivergence.c -output bvDivergence_mex
        mex CFLAGS="\$CFLAGS -std=c99" bvBasicFunctions.o -I'./bvBasicLib' bvSrc_C/deconvolutionPrimalDual.c -output deconvolutionPrimalDual_mex
        
        !nvcc -c bvSrc_CUDA/bvProximalOperatorGPU.cu -Xcompiler -fPIC -I /opt/local/matlab/extern/include/
        mex CFLAGS="\$CFLAGS -std=c99" bvProximalOperatorGPU.o -L /opt/local/NVIDIA_GPU_Computing_SDK/CUDALibraries -lcudart  -output bvProximalOperator_mex
        
        !nvcc -c bvSrc_CUDA/inpaintingPrimalDualGPU.cu -Xcompiler -fPIC -I /opt/local/matlab/extern/include/
        mex CFLAGS="\$CFLAGS -std=c99" inpaintingPrimalDualGPU.o -cxx -L /opt/local/NVIDIA_GPU_Computing_SDK/C/common/inc -lcudart -output inpaintingPrimalDual_mex
        
        !nvcc -c bvSrc_CUDA/TVL1_GPU.cu -Xcompiler -fPIC -I /opt/local/matlab/extern/include/
        mex CFLAGS="\$CFLAGS -std=c99" TVL1_GPU.o -L /usr/local/cuda/lib64 -lcudart -lcufft -output TVL1_mex
        
        
        % Si tout va bien :
        disp(' ')
        disp('Installation complete successfull !')
        disp(' ')
        disp('Vous disposez desormais des fonctions suivantes :')
        disp('bvGradient ;')
        disp('bvDivergence ;')
        disp('bvProximalOperator (en version paralelisee) ;')
        disp('inpaintingPrimalDual (en version paralelisee) ;')
        disp('deconvolutionPrimalDual ;')
        disp('TVL1_mex (en version paralelisee).')
        disp(' ')
        disp('Amusez-vous bien !')
    end    
elseif ispc
        clc
        
        addpath(pwd);
        cd ./bvM_Files
        addpath(pwd);
        cd ..
        cd ./bvData
        addpath(pwd);
        cd ..
        savepath;
        
        disp('Installation start ...')
        disp(' ')
        
        % compilation du fichier des fonctions de base.
       % A=[pwd,'\bvBasicLib'];
        mex  -c -I[pwd filesep 'bvBasicLib' filesep ] ./bvBasicLib/bvBasicFunctions.c
        
        % compilation des fonctions.
        mex('-v', 'bvBasicFunctions.obj', ['-I',pwd filesep 'bvBasicLib' filesep],  './bvSrc_C/bvGradient.c', '-output', 'bvGradient_mex');
        mex('-v', 'bvBasicFunctions.obj', ['-I',pwd filesep 'bvBasicLib' filesep], './bvSrc_C/bvDivergence.c', '-output', 'bvDivergence_mex');
        mex('-v', 'bvBasicFunctions.obj', ['-I',pwd filesep 'bvBasicLib' filesep],  './bvSrc_C/bvDivergence.c', '-output', 'bvDivergence_mex');
        mex('-v', 'bvBasicFunctions.obj', ['-I',pwd filesep 'bvBasicLib' filesep],  './bvSrc_C/deconvolutionPrimalDual.c', '-output', 'deconvolutionPrimalDual_mex');
    if nargin==0
        mex('-v', 'bvBasicFunctions.obj', ['-I',pwd filesep 'bvBasicLib' filesep],  'bvSrc_C/bvProximalOperator.c', '-output', 'bvProximalOperator_mex');
        mex('-v', 'bvBasicFunctions.obj', ['-I',pwd filesep 'bvBasicLib' filesep],  'bvSrc_C/inpaintingPrimalDual.c', '-output', 'inpaintingPrimalDual_mex');
        mex('-v', 'bvBasicFunctions.obj', ['-I',pwd filesep 'bvBasicLib' filesep],  'bvSrc_C/TVL1_mex.c', '-output', 'TVL1_mex');
        
        % Si tout va bien :
        disp(' ')
        disp('Installation complete successfull !')
        disp(' ')
        disp('Vous disposez desormais des fonctions suivantes :')
        disp('bvGradient')
        disp('bvDivergence')
        disp('bvProximalOperator')
        disp('inpaintingPrimalDual')
        disp('deconvolutionPrimalDual')
        disp('TVL1_mex')
        disp(' ')
        disp('Amusez-vous bien !')
   elseif  strcmp(arg, 'gpu')
        
        !nvcc -c bvSrc_CUDA/bvProximalOperatorGPU.cu -Xcompiler -fPIC -I"D:\Program Files\MATLAB\R2011a\extern\include"
        mex bvProximalOperatorGPU.obj...
            -I'C:\Program Files\NVIDIA GPU Computing Toolkit\CUDA\v5.0\include\'...
            -L'C:\Program Files\NVIDIA GPU Computing Toolkit\CUDA\v5.0\lib\Win32\'...
            -lcudart -lcufft -output bvProximalOperator_mex
        
        !nvcc -c bvSrc_CUDA/inpaintingPrimalDualGPU.cu -Xcompiler -fPIC -I"D:\Program Files\MATLAB\R2011a\extern\include"
        mex inpaintingPrimalDualGPU.obj...
            -I'C:\Program Files\NVIDIA GPU Computing Toolkit\CUDA\v5.0\include\'...
            -L'C:\Program Files\NVIDIA GPU Computing Toolkit\CUDA\v5.0\lib\Win32\'...
           -lcudart -lcufft  -output inpaintingPrimalDual_mex
        
        !nvcc -c bvSrc_CUDA/TVL1_GPU.cu -Xcompiler -fPIC -I"D:\Program Files\MATLAB\R2011a\extern\include"
        mex TVL1_GPU.obj...
            -I'C:\Program Files\NVIDIA GPU Computing Toolkit\CUDA\v5.0\include\'...
            -L'C:\Program Files\NVIDIA GPU Computing Toolkit\CUDA\v5.0\lib\Win32\'...
            -lcudart -lcufft -output TVL1_mex
        
        
        % Si tout va bien :
        disp(' ')
        disp('Installation complete successfull !')
        disp(' ')
        disp('Vous disposez desormais des fonctions suivantes :')
        disp('bvGradient ;')
        disp('bvDivergence ;')
        disp('bvProximalOperator (en version paralelisee) ;')
        disp('inpaintingPrimalDual (en version paralelisee) ;')
        disp('deconvolutionPrimalDual ;')
        disp('TVL1_mex (en version paralelisee).')
        disp(' ')
        disp('Amusez-vous bien !')
   
    
elseif  strcmp(arg, 'clear')
        % nettoyage complet
        clc, clear all
        
        list=dir;
        for i=3:length(list)
            fname= list(i).name;
            if (strcmp(fname(end-1:end),'.c') || ...
                    strcmp(fname(end-1:end),'.h')|| ...
                    strcmp(fname(end-1:end),'.m')|| ...
                    strcmp(fname(end-2:end),'.cu')|| ...
                    strcmp(fname(end-3:end),'.mat')|| ...
                    strcmp(fname(end-3:end),'.bat')|| ...
                    isdir(fname))
                
            else
                delete(list(i).name);
            end
        end
        rmpath(pwd);
        cd ./bvM_Files
        rmpath(pwd);
        cd ..
        cd ./bvData
        rmpath(pwd);
        cd ..
        
    end
end
cd ..
